import json

import pendulum
import pytest
from aiohttp import ClientSession
from yarl import URL

from resa_padel.gestion_sports.gestion_sports_connector import GestionSportsConnector
from resa_padel.models import User, BookingFilter

gestion_sports_url = "https://toulousepadelclub.gestion-sports.com"
test_user = "padel.testing@jouf.fr"
test_user_id = "232382"
test_password = "ridicule"
test_club_id = "88"
user = User(login=test_user, password=test_password, club_id=test_club_id)
test_court_id = "607"

now = pendulum.now()
date_to_book = now.add(days=6)
datetime_to_book = date_to_book.set(hour=18, minute=0, second=0)
padel_id = 217
court_id = 607


@pytest.mark.asyncio
async def test_should_connect_to_gestion_sports_website():
    async with ClientSession() as session:
        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("PHPSESSID") is None
        gs_connector = GestionSportsConnector(session, gestion_sports_url)

        response = await gs_connector.connect()

        assert response.status == 200
        assert response.request_info.method == "GET"
        assert response.content_type == "text/html"
        assert response.request_info.url == URL(gestion_sports_url + "/connexion.php")
        assert response.charset == "UTF-8"

        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("PHPSESSID") is not None


@pytest.mark.asyncio
async def test_should_login_to_gestion_sports_website():
    async with ClientSession() as session:
        gs_connector = GestionSportsConnector(session, gestion_sports_url)
        await gs_connector.connect()

        response = await gs_connector.login(user)

        assert response.status == 200
        assert response.request_info.url == URL(gestion_sports_url + "/connexion.php")
        assert response.request_info.method == "POST"

        cookies = session.cookie_jar.filter_cookies(URL(gestion_sports_url))
        assert cookies.get("COOK_ID_CLUB").value == test_club_id
        assert cookies.get("COOK_ID_USER").value == test_user_id
        assert cookies.get("PHPSESSID") is not None


@pytest.mark.asyncio
async def test_should_book_a_court_from_gestion_sports():
    async with ClientSession() as session:
        gs_connector = GestionSportsConnector(session, gestion_sports_url)
        await gs_connector.connect()
        await gs_connector.login(user)

        booking_filter = BookingFilter(
            court_id=court_id, sport_id=padel_id, date=datetime_to_book
        )
        response = await gs_connector.book(booking_filter)

        assert response.status == 200
        assert response.request_info.url == URL(
            gestion_sports_url + "/membre/reservation.html?"
        )
        assert response.request_info.method == "POST"

        result = await response.text()
        formatted_result = result.removeprefix('"').removesuffix('"')
        result_json = json.loads(formatted_result)
        # booking any day at 18:00 should always be an error if everything goes well ;)
        assert result_json["status"] == "error"
